(* Copyright 2011 Cornell University
 * Copyright 2012 Cornell University
 *
 *
 * This file is part of EventML - a tool aiming at specifying
 * distributed protocols in an ML like language.  It is an interface
 * to the logic of events and is compiled into Nuprl.  It is written
 * by the NUPRL group of Cornell University, Ithaca, NY.
 *
 * EventML is a free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * EventML is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with EventML.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  o Authors:     Vincent Rahli
 *  o Affiliation: Cornell University, NUPRL group
 *  o Date:        20 May 2011
 *  o File name:   Evaluator3.sml
 *  o Description: uses continuations.
 *)


structure Evaluator3 = struct

structure B   = Tools
structure T   = NuprlTerms
structure P   = Primitive
structure E   = EvalProperties
structure EV2 = Evaluator2

val NextStepEval3 = EV2.NextStepEval2
val clos_lst      = EV2.clos_lst

fun unit x s cont = cont x s
fun bind (m, k) = fn s => fn cont => m s (fn v => fn s' => k v s' cont)
val >>= = bind
infix >>=
(*fun m_bind m k = fn cl => fn s => let val (v,s') = m s in k v cl s' end
val >>>= = m_bind
infix >>>=*)

fun m_num_principals opid term (state as (steps,_)) K =
    if steps < 0
       andalso opid = "apply"
       andalso T.is_nuprl_term "ycomb" (#1 (T.dest_apply term))
    then K 0 state
    else K (E.num_principals opid) state

(* EVALUATOR3 - continuations *)
fun Evaluator3' term state =

    let fun EvalList cbva [] env = unit []
	  | EvalList cbva ((vars,t)::rest) env =
	    if List.null vars
	    then (Eval (t,env))
		     >>=
		     (fn p => ((if cbva then EvalAll else unit) p)
				  >>=
				  (fn q => (EvalList cbva rest env)
					       >>=
					       (fn vs => unit (q :: vs))))
	    else raise Fail "term has bound variables"

	and Eval (t, env) =
	    let val (opr as (opid,params),subterms) = T.dest_term t
	    in (m_num_principals opid t)
		   >>=
		   (fn n =>
		       let val (p,np) = B.split n subterms
		       in (EvalList (E.is_eval_all opid) p env)
			      >>=
			      (fn c =>
			       fn (state as (n,(lib,cls))) =>
			       fn K =>
				  let val ((t',e',ev),state') = NextStepEval3 cls lib t c np env state
				  in if ev then Eval (t',e') state' K else K (t',e') state'
				  end handle _ => K (T.mk_nuprl_term opr ((clos_lst c) @ np), env) state)
		       end)
	    end

	and EvalAll (t, env) =
	    let val (opr as (opid,params),subterms) = T.dest_term t
		val (principals,nprincipals) = B.split (E.num_principal_all opid) subterms
		val env' = if List.null nprincipals then T.mk_nuprl_empty_env_term else env
	    in (EvalList true principals env)
		   >>=
		   (fn c => unit (T.mk_nuprl_term opr ((clos_lst c) @ nprincipals), env'))
	    end

	fun K p state = (p, state)

    in if T.is_nuprl_iclosure_term term
       then Eval (T.dest_iclosure term) state K
       else Eval (term, T.mk_nuprl_empty_env_term) state K
    end

fun Evaluator3 cls lib steps term =
    let val ((t',e'),(n,s)) = Evaluator3' term (steps, (lib,cls))
	val answer =
	    if cls andalso T.is_nuprl_pair_term t'
	    then let val (s,msgs) = T.dest_pair 8 t'
		     val msgs' = T.closure2term msgs e'
		 in T.mk_nuprl_pair_term (T.mk_nuprl_iclosure_term s e') msgs'
		 end
	    else T.closure2term t' e'
    in (answer, steps - n)
    end

end
