(* Copyright 2011 Cornell University
 *
 *
 * This file is part of EventML - a tool aiming at specifying
 * distributed protocols in an ML like language.  It is an interface
 * to the logic of events and is compiled into Nuprl.  It is written
 * by the NUPRL group of Cornell University, Ithaca, NY.
 *
 * EventML is a free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * EventML is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with EventML.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  o Authors:     Vincent Rahli
 *  o Affiliation: Cornell University, NUPRL group
 *  o Date:        29 July 2011
 *  o File name:   Tools.sml
 *  o Description: Some useful functions.
 *)


structure Tools :> TOOLS = struct

fun member (element : string) list =
    List.exists (fn v => v = element) list

fun apply_alist [] _ = raise Fail "apply_alist"
  | apply_alist ((x : string, value ) :: tail) elt =
    if x = elt
    then value
    else apply_alist tail elt

fun can f x = (f x; true) handle _ => false

fun fst (x, y) = x
fun snd (x, y) = y

fun mapfilter f list =
    List.mapPartial (fn x => ((SOME (f x)) handle _ => NONE))
		    list

fun decr_steps steps =
    if steps = 0
    then raise Fail "decr_steps"
    else steps - 1

fun split n list = (List.take (list, n), List.drop (list, n))

end
